# w.c.s. - web application for online forms
# Copyright (C) 2005-2010  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import sys
import time
import traceback
import uuid

from quixote import get_publisher, get_response
from quixote.directory import Directory

from . import _, N_, errors
from .storage import StorableObject


class AfterJobStatusDirectory(Directory):
    def _q_lookup(self, component):
        try:
            job = AfterJob.get(component)
        except KeyError:
            raise errors.TraversalError()

        response = get_response()
        response.set_content_type('text/plain')
        if not job.completion_status:
            return job.status + '|' + _(job.status)
        return job.status + '|' + _(job.status) + ' ' + job.completion_status


class AfterJob(StorableObject):
    _names = 'afterjobs'
    _reset_class = False

    label = None
    status = None
    creation_time = None
    completion_time = None
    completion_status = None

    execute = None

    def __init__(self, label=None, cmd=None, **kwargs):
        super().__init__(id=str(uuid.uuid4()))
        if label:
            self.label = label
        self.creation_time = time.time()
        self.job_cmd = cmd
        self.status = N_('registered')
        self.kwargs = kwargs

    def run(self, spool=False):
        if self.completion_time:
            return

        if spool and self.id and self.execute:
            from django.conf import settings
            if 'uwsgi' in sys.modules and settings.WCS_MANAGE_COMMAND:
                from .spooler import run_after_job
                self.store()
                run_after_job.spool(tenant_dir=get_publisher().app_dir, job_id=self.id)
                return

        self.status = N_('running')
        if self.id:
            self.store()
        try:
            if self.execute:
                self.execute()
            else:
                self.job_cmd(job=self)
        except Exception:
            get_publisher().notify_of_exception(sys.exc_info())
            self.exception = traceback.format_exc()
            self.status = N_('failed')
        else:
            self.status = N_('completed')
        self.completion_time = time.time()
        if self.id:
            self.store()

    def __getstate__(self):
        if not isinstance(self.job_cmd, str):
            obj_dict = self.__dict__.copy()
            obj_dict['job_cmd'] = None
            return obj_dict
        return self.__dict__
